
#ifndef h_pnobridge_h
#define h_pnobridge_h

#undef  VERSION

#define VERSION_MAJOR "1"
#define VERSION_MINOR "1"
#define VERSION "1.1"
#define VERSION_DB 0x0101  //0xMMmm  M Major, m minor

/***********************
/* 0 => standard 320x320
/* 1 => hires+ device
/* 2 => clie device
/* 4 => zodiac
/***********************/
#define _ZODIAC_DEVICE_ 4
#define _CLIE_DEVICE_ 2
#define _HIRES_PLUS_DEVICE_ 1


#define FTR_PNO 0
#define FTR_WAD_FILENAME_ID 1
#define FTR_VIDEO_RESX_ID 2
#define FTR_VIDEO_RESY_ID 3
#define FTR_VIDEO_ROT_ID 4
#define FTR_OPT_INPUTCONF_ID 5
#define FTR_PWAD_FILENAME_ID 6
#define FTR_MUSIC_MODE_ID 7

#define FTR_MIDI_START_ID 100


#define FORM_LISTFILE_ID 5000
#define FORM_LISTPATCHFILE_ID 5001

#define BTN_BL 0x11110000
#define BTN_ML 0x22220000
#define BTN_TL 0x44440000
#define BTN_BR 0x00001111
#define BTN_MR 0x00002222
#define BTN_TR 0x00004444
#define BTN_TM 0x00008888
#define BTN_BM 0x88880000



// register your own at http://www.palmos.com/dev/creatorid/
/*#define appName					"ZHexen"
#define appFileCreator			'ZHXN'
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01*/

#define CREATORID 'ZHXN'
#define VFSPATH "/PALM/Programs/ZHexen/"
#define VFSROOT "/PALM/Programs/"

#define WELCOME_MSG "Welcome to ZHexen 1.1"

// 240*160 -> 480*320
#define _PEN_ADJUST_FACTORX_ 2
#define _PEN_ADJUST_FACTORY_ 2

#define AUDIO_RATE 44100
#define AUDIO_FPS     60
#define AUDIO_STEREO   2 /* or 1 for mono */
#define AUDIO_WIDTH    2 /* or 1 for 8bit */
#define AUDIO_BUFSIZE_BYTES (AUDIO_RATE/AUDIO_FPS*AUDIO_STEREO*AUDIO_WIDTH)
#define AUDIO_BUFSIZE_FRAMES (AUDIO_RATE/AUDIO_FPS)
#define AUDIO_FMT      sndInt16Little

//#define AUDIO_ARM      1 /* if audio-arm defined, then do arm callback */
//#define AUDIO_VOLUME 20 /* see audio_volume(); *64 */

// debug in pno?
//#define PNODEBUG 1

typedef struct {
  UInt32 vram;    // address of screen VRAM
  UInt32 eventf;  // function pointer to 68k func to do event handling
  UInt32 display_pitch;  // screenbuffer width
  UInt32 display_width;  // X width of display
  UInt32 display_height; // Y height of display
  UInt32 param1;       // pointer to chosen game rom filename
  UInt32 param2;      // pointer to audio buffer
  UInt32 flags;          // flags for pnolet
} pnobridge_t;

// for requesting data from the 68k side, from the pnolet
//#define BRIDGESHIFT 5
//#define BRIDGEMASK  0x1F
enum {
  BRIDGE_MemGluePtrNew = 1,
  BRIDGE_debugu32, 
  BRIDGE_printfu32,
  BRIDGE_accum,
  BRIDGE_debugs,
  BRIDGE_printfs,
  BRIDGE_error,
  BRIDGE_progress,
  BRIDGE_setaudio,
  BRIDGE_bigalloc,
  BRIDGE_get_skip,
  BRIDGE_bigfree,
  BRIDGE_zodiacblit,
  BRIDGE_max,
  BRIDGE_SndStreamCreate,
  BRIDGE_SndStreamDelete,
  BRIDGE_SndStreamStart,
  BRIDGE_SndStreamStop,
  BRIDGE_SndStreamGetVolume,
  BRIDGE_SndStreamSetVolume,
  BRIDGE_Fullscreen,
  BRIDGE_consoleCmd
};

// bridge options going into the pno
#define BF_FULLSCREEN 1
#define BF_AUDIO      2
#define BF_LOWMEM     4
#define BF_ZODIAC     8
#define BF_HIRESPLUS  16
#define BF_PLAYMOD    16
#define BF_STOPMOD    32
#define BF_INITMOD    64
#define BF_FREEMOD    128

#define BF_UNZIP	  1

#define BF_HASPIN	  256

// for calling back from pno to host
typedef struct {
  UInt32 a;
  UInt32 b;
  UInt32 c;
  UInt32 d;
} hostcall_t;

typedef struct {
  UInt32 a;
  UInt32 b;
  UInt32 c;
  UInt32 d;
  UInt32 e;
  UInt32 f;
  UInt32 g;
  UInt32 h;
  UInt32 i;
} bighostcall_t;

// returning status from event check; buttons and other stuff..
enum {
  STAT_left = 1,
  STAT_right = 2,
  STAT_up = 4,
  STAT_down = 8,
  STAT_fire1 = 16,
  STAT_fire2 = 32,
  STAT_exeunt = 64,
  STAT_pause = 128,
  STAT_start = 256,
  STAT_skip = 512,
  STAT_max
};

// go to 68k side and ask it to do events, returning the goods
UInt32 host_events ( UInt32 foo );
void *GetObjectPtr ( UInt16 objectID );

extern pnobridge_t *g_pnobridge;

#define FLUSH_D_CACHE {int i,j;UInt8 *tmpPtr;\
		tmpPtr=MemPtrNew(32768);j=0;\
		for (i=0;i<32768;i++) j+=(int)tmpPtr[i];\
		MemPtrFree(tmpPtr);tmpPtr=MemPtrNew(32768);\
		j=0;for (i=0;i<32768;i++) j+=(int)tmpPtr[i];\
		MemPtrFree(tmpPtr);\
	}

typedef struct {
	UInt32	g_inputconf[30];
	Char g_iwad_name[256];
	Char g_pwad_name[256];
	Char g_pwad_active;
	Char g_fullscreen;
	Char first;
	Char key_defined;
	Char g_music_mode;
} zhexen_pref_t;


#endif
