
#include <Tapwave.h>
//#include <PalmOS.h>
#include <VfsMgr.h>
//#include <PalmOSGlue.h>
#include "AppRsc.h"

#include "endianutils.h"
#include "host68k.h"

#include "zodiac.h"

TwGfxType *lib = NULL;
TwGfxInfoType ginfo;



UInt8 g_zodiac = 0;
TwGfxSurfaceType *g_display_surface;
TwGfxSurfaceType *g_backbuffer_surface;

UInt8 tw_gfx_init ( void ) {
  Err error;
  TwGfxSurfaceInfoType surfInfo;

  // prepare TW necessities..


  // prepare for call
  ginfo.size = sizeof(ginfo);

  // set up the API
  error = TwGfxOpen ( &lib, &ginfo );

  if ( error ) {
    return ( 1 );
  }

  // get physical display surface
  error = TwGfxGetDisplaySurface ( lib, &g_display_surface );

  if ( error ) {
    return ( 2 );
  }

  // create a backbuffer surface (to blit from)
  surfInfo.size = sizeof(surfInfo);
  surfInfo.width = 480;//ginfo.displayWidth;
  surfInfo.height = 320;//ginfo.displayHeight;
  surfInfo.location = twGfxLocationAcceleratorMemory;
  surfInfo.pixelFormat = twGfxPixelFormatRGB565;
  error = TwGfxAllocSurface ( lib, &g_backbuffer_surface, &surfInfo );

  if ( error ) {
    return ( 3 );
  }

  return ( 0 );
}

void tw_gfx_close ( void ) {

  // free backbuffer
  TwGfxFreeSurface ( g_backbuffer_surface );

  // release lib
  TwGfxClose ( lib );

  return;
}

void tw_gfx_blit ( UInt16 w,UInt16 h ) {
  Err error;

  // bounds
  TwGfxPointType zeropoint;
  TwGfxRectType rect;   // source

  zeropoint.x = (w-ginfo.displayWidth)>>1;
  zeropoint.y = (h-ginfo.displayHeight)>>1;

  rect.x = 0;
  rect.y = 0;
  rect.w = w;
  rect.h = h;

  // blit backbuffer to screen
  // blit backbuffer to screen
  TwGfxAsyncBlt ( g_display_surface, &zeropoint,
		     g_backbuffer_surface, &rect);  
  return;
}

void tw_gfx_blit2 ( UInt16 width, UInt16 height, UInt16 scale_smooth ) {
  Err error;

  // bounds
  TwGfxPointType zeropoint;
  TwGfxRectType rect;   // source
  TwGfxRectType drect;  // destination

  zeropoint.x = 0;
  zeropoint.y = 0;

  rect.x = 0;
  rect.y = 0;
  rect.w = width;
  rect.h = height;

  switch (scale_smooth&3){
  	case 0:	  
	  drect.w = width*5/4;
	  drect.h = height*5/4;
	  break;
	case 1:
   	  drect.w = 480;//ginfo.displayWidth;
	  drect.h = 320;//ginfo.displayHeight;
	  break;
	case 2:
	  drect.w = 160*2;
	  drect.h = 144*2;
	  break;
	case 3:
	  drect.w = 224*2;//ginfo.displayWidth;
	  drect.h = 144*2;//ginfo.displayHeight;
	  break;
  }


  drect.x = (ginfo.displayWidth-drect.w)>>1;
  drect.y = (ginfo.displayHeight-drect.h)>>1;
  if (scale_smooth&4)  TwGfxStretchBlt2 ( g_display_surface, &drect,
		     g_backbuffer_surface, &rect,
		     twGfxStretchFast | twGfxStretchSmooth );
	else  TwGfxStretchBlt2 ( g_display_surface, &drect,
		     g_backbuffer_surface, &rect,
		     twGfxStretchFast  );

  return;
}

Boolean TwCheckAPIVersion ( UInt32 minVersion ) {
  UInt32 version;
  FtrGet ( twFtrCreator, twFtrAPIVersion, &version );
  return ( version >= minVersion );
}

UInt8 is_zodiac ( void ) {

  if ( TwCheckAPIVersion ( TAPWAVE_API_VERSION ) ) {
    return ( 1 );
  }

  return ( 0 );
}

UInt32 tw_gfx_lock ( void ) {
  void *x;
  Err error;

  error = TwGfxLockSurface ( g_backbuffer_surface, &x );

  if ( error ) {
  	DEBUGS("Cannot lock surface");
  	DEBUGU32(error);
    return ( NULL );
  }

  return ( (UInt32) x );
}

void tw_gfx_unlock ( void ) {
  TwGfxUnlockSurface ( g_backbuffer_surface, true );
  return;
}
